/*************************************************************
                WHOIS CLIENT ON REXX
      autor: Dmitry Maximovich <maxim@pabl.ru>

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
**************************************************************/

/* setting default port number and server name for whois service */
nPortNumber	=	43
sServer		=	'whois.internic.net'

/* parsing arguments */
PARSE VALUE ARG(1) WITH A1 A2 A3

IF A1 = '' THEN
   DO    
     SAY 'USAGE: whois [-h hostname] name'
     EXIT
   END

IF A1 = '-h' THEN
  DO
    sServer = A2
    sName = A3
  END
ELSE
  sName = A1

/* Load REXX Socket library if not already loaded       */
IF RxFuncQuery("SockLoadFuncs") THEN
 DO
   rc = RxFuncAdd("SockLoadFuncs","rxSock","SockLoadFuncs")
   rc = SockLoadFuncs()
 END

/* Load REXX Utility library if not already loaded       */
IF RxFuncQuery("SysLoadFuncs") THEN
 DO
   rc = RxFuncAdd( "SysLoadFuncs","RexxUtil","SysLoadFuncs")
   rc = SysLoadFuncs()
 END

/* resolwing host address */
rc=SockGetHostByName(sServer,"host.")

IF rc <> 1 THEN
   DO
     SAY 'CANNOT RESOLVE HOSTNAME TO ADDRESS: 'sServer
     EXIT -1
   END

SAY 'Used whois server:'host.name', address:'host.addr

/* creating a socket */
socket = SockSocket('AF_INET','SOCK_STREAM',0)
IF socket < 0 THEN
   DO
     SAY 'UNABLE TO CREATE A SOCKET'
     EXIT -1
   END
 
address.family = 'AF_INET'
address.port = nPortNumber
address.addr = host.addr

/* connecting a socket */
rc = SockConnect(socket,'address.')
IF rc < 0 THEN
   DO
     SAY 'UNABLE TO CONNECT TO SERVER:'address.addr
     SIGNAL DO
   END

/* sending request */
rc = SockSend(socket, sName||"0D0A"x)
 
/* receive ansver */
Resp = ''
DO FOREVER
   rc = SockRecv(socket,"sReceive",256)
   IF rc <= 0 THEN
     LEAVE
   Resp = Resp || sReceive
END

SAY ''

/* CR -> CRLF */
nStart = 1
nStop = pos(X2C("0A"), Resp)
do while nStop > 0
  SAY SUBSTR(Resp,nStart,nStop-nStart)
  nStart = nStop + 1
  nStop = pos(X2C("0A"), Resp, nStart)
end

 
DO:
rc = SockShutDown(socket,2)
rc = SockClose(socket)

